-----------------------------------------------------------------------------------------------
-- Client Lua Script for NexusPanel
-- Copyright (c) NCsoft. All rights reserved
-- developed by Florian Franke
local actVersion = "1.5.1"
local stop=false
-----------------------------------------------------------------------------------------------
 
require "Window"
-----------------------------------------------------------------------------------------------
-- NexusPanel Module Definition
-----------------------------------------------------------------------------------------------
local NexusPanel = {} 

local time = os.time

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local firstTime = true --vorrbergehend

local ktClassIcon =
{
	[GameLib.CodeEnumClass.Medic] 			= "Icon_Windows_UI_CRB_Medic",
	[GameLib.CodeEnumClass.Esper] 			= "Icon_Windows_UI_CRB_Esper",
	[GameLib.CodeEnumClass.Warrior] 		= "Icon_Windows_UI_CRB_Warrior",
	[GameLib.CodeEnumClass.Stalker]			= "Icon_Windows_UI_CRB_Stalker",
	[GameLib.CodeEnumClass.Engineer] 		= "Icon_Windows_UI_CRB_Engineer",
	[GameLib.CodeEnumClass.Spellslinger] 	= "Icon_Windows_UI_CRB_Spellslinger",
}

local karCurrency =
{						
	{eType = Money.CodeEnumCurrencyType.Renown, 			strTitle = Apollo.GetString("CRB_Renown"), 				strDescription = Apollo.GetString("CRB_Renown_Desc")},
	{eType = Money.CodeEnumCurrencyType.ElderGems, 			strTitle = Apollo.GetString("CRB_Elder_Gems"), 			strDescription = Apollo.GetString("CRB_Elder_Gems_Desc")},
	{eType = Money.CodeEnumCurrencyType.Glory, 			strTitle = Apollo.GetString("CRB_Glory"), 			strDescription = Apollo.GetString("CRB_Glory_Desc")},
	{eType = Money.CodeEnumCurrencyType.Prestige, 			strTitle = Apollo.GetString("CRB_Prestige"), 			strDescription = Apollo.GetString("CRB_Prestige_Desc")},
	{eType = Money.CodeEnumCurrencyType.CraftingVouchers, 	strTitle = Apollo.GetString("CRB_Crafting_Vouchers"), 	strDescription = Apollo.GetString("CRB_Crafting_Voucher_Desc")},
}
		
local knMaxLevel = 50

--SUBPANELS/WIDGETS

local kSubPanelNames =
{
	"TimePanel",
	"LocationPanel",
	"SocialPanel",
	"RepairPanel",
	"LevelTimePanel",
	"CurrencyPanel",
	"TechInfoPanel",
	"MoneyPanel",
	"InventoryPanel",
	"CraftingPanel",
}
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function NexusPanel:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	o.config = {}
	o.config.subZone = {}
	o.config.subZone.name = ""
	o.config.subZone.id = -1
	o.config.tWindowLocation = {}
	o.config.chkbBGTransparency = false
	o.config.chkbBordersTransparency = false
	o.config.chkbShowModeMouseOver = false
	o.config.chkbShowModeInCombat = true
	o.config.chkbShowModeOutOfCombat = true
	o.config.chkbLockPanel = false
	o.config.panel = {}
	o.config.lastLogin = nil
	o.config.version = ""
	o.config.chkbHideWSCurrency = false
	o.config.layer = 
	{
		["LocationPanel"] = 0,
		["TimePanel"] = 0,
		["MoneyPanel"] = 0,
		["SocialPanel"] = 0,
		["TechInfoPanel"] = 0,
		["RepairPanel"] = 0,
		["LevelTimePanel"] = 0,
		["CraftingPanel"] = 0,
	}
	
    return o
end

function NexusPanel:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
	Apollo.RegisterSlashCommand("np","OnSlashCommand",self)
	Apollo.CreateTimer("NPTimer",1,true)
end
 

-----------------------------------------------------------------------------------------------
-- NexusPanel OnLoad
-----------------------------------------------------------------------------------------------
function NexusPanel:OnLoad()
	Apollo.RegisterEventHandler("SubZoneChanged","OnSubZoneChanged", self)
	Apollo.RegisterEventHandler("ExperienceGained","OnExperienceGained", self)
	Apollo.RegisterEventHandler("ElderPointsGained","OnElderPointsGained", self)
	Apollo.RegisterEventHandler("QuestStateChanged", 			"OnQuestStateChanged", self)
	--Apollo.RegisterEventHandler("LootedMoney","OnLootedMoney", self)
	Apollo.RegisterTimerHandler("NPTimer","OnTimerTick", self)
	Apollo.RegisterEventHandler("PlayedTime", "OnPlayedTime", self)
	Apollo.RegisterEventHandler("ShowOptions", "ShowOptions", self)
	Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
	Apollo.RegisterEventHandler("WindowManagementReady","OnWindowManagementReady",	self)
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("NexusPanel.xml")
	--self.wndMain = Apollo.LoadForm(self.xmlDoc, "Panel", nil, self)
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	
	--VARIABLES
	self.data = {}
	
	self.data.nLevel = 0
	
	self.data.nCurrentXP = nil
	self.data.nNeededXPTotal = nil			
	self.data.nNeededXP = nil
	self.data.nRestedXP = nil							
	self.data.nRestedXPPool = nil
	
	self.data.totalXPGainedThisSession = 0
	self.data.timeForLevelUp = 0 --hours
	
	self.data.moneyAtBegin = 0 -- Copper
	self.data.totalMoneyLootedThisSession = nil --Copper
	
	self.data.actFPS = 0.0
	self.data.totalFPS = 0.0
	self.data.fpsPerSec = 0.0
	
	self.data.actLatency = 0
	self.data.totalLatency = 0
	self.data.averageLatency = 0
	
	self.data.actPingTime = 0
	self.data.totalPingTime = 0
	self.data.averagePingTime = 0
	
	self.data.totalTicks = 0
	self.data.xpPerTick = 0
	self.data.xpperHour = 0
	self.data.nSecondsPlayed = 0
	self.data.nLevelStart = 0
	self.data.nSessionStart = time()
	
	self.data.tPos = {}
	self.data.currentZone = nil
	
	self.data.gametime = 0
	self.data.nTimeThisSession = 0
	self.data.nTimeThisLevel = 0
	self.data.nTotalTimePlayed = 0
	
	self.data.tReputationLevels = GameLib.GetReputationLevels()
	
	self.data.repairCosts = 0
	
	self.data.elderPoints = 0
	self.data.elderPointsperGem = 0
	self.data.elderPointsPerTick = 0
	self.data.elderPointsperHour = 0
	self.data.totalElderPointsGainedThisSession = 0
	self.data.timeForNextGem = 0
	self.data.nCurrentElderPoints = nil		
	self.data.nNeededElderPoints = nil
	self.data.weeklyElderPoints = GetPeriodicElderPoints()
	self.data.strWeeklyElderPoints = ""
	self.data.weeklyElderPointsCap = GameLib.ElderPointsDailyMax
	
	self.data.localTime = nil
	self.data.gameTime = nil
	self.data.serverTime = nil
	self.data.strCoords = ""
	
	self.data.nOnlineFriends = 0
	self.data.nOnlineAccountFriends = 0
	self.data.onlineFriends = {}
	self.data.onlineAccountFriends = {}
	self.data.nOnlineFriendsTotal = 0
	
	self.data.tGuild = nil
	self.data.nOnlineGuildMembers = 0
	
	self.data.circles = {}
	
	self.data.nUnreadMessages = 0
	
	self.data.actRepairPercent = 0
	self.data.tRepairItems = {}
	
	self.data.nTotalInventorySlots = 0
	self.data.nEmptyInventorySlots = 0
	self.data.nUsedInventorySlots = 0
	
	self.data.nQuestsCompleted = 0
	self.data.nQuestsAchieved = 0
	self.data.nTotalQuestsXPInSession = 0
	
	self.data.craftingXP = {}
	
	self.data.tLayerData = 
	{
		["LocationPanel"] = 
		{
			["Index"] = 0,
			["NextTooltip"] =
			{
				[1] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">zone with coordinates</T> instead of <T TextColor=\"ChannelZone\">subzone with coordinates</T>.</A></P>",
				[2] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">subzone</T> instead of <T TextColor=\"ChannelZone\">zone with coordinates</T>.</A></P>",
				[3] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">zone</T> instead of <T TextColor=\"ChannelZone\">subzone</T>.</A></P>",
				[4] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">subzone with coordinates</T> instead of <T TextColor=\"ChannelZone\">zone</T>.</A></P>",
			},
			["Value"] =
			{
				[1] = "",
				[2] = "",
				[3] = "",
				[4] = "",
			},
		},
		["TimePanel"] = 
		{
			["Index"] = 0,
			["NextTooltip"] =
			{
				[1] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">server time 24-hour clock</T> instead of <T TextColor=\"ChannelZone\">local time 24-hour clock</T>.</A></P>",
				[2] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">local time 12-hour clock</T> instead of <T TextColor=\"ChannelZone\">server time 24-hour clock</T>.</A></P>",
				[3] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">server time 12-hour clock</T> instead of <T TextColor=\"ChannelZone\">local time 12-hour clock</T>.</A></P>",
				[4] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">local time 24-hour clock</T> instead of <T TextColor=\"ChannelZone\">server time 12-hour clock</T>.</A></P>",
			},
			["Value"] =
			{
				[1] = "",
				[2] = "",
				[3] = "",
				[4] = "",
			},
			["Size"] =
			{
				[1] ={["w"] = 71,["h"] = 28,},
				[2] ={["w"] = 71,["h"] = 28,},
				[3] ={["w"] = 91,["h"] = 28,},
				[4] ={["w"] = 91,["h"] = 28,},
			},
		},
		
		["MoneyPanel"] = 
		{
			["Index"] = 0,
			["NextTooltip"] =
			{
				[1] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">money amount</T> instead of <T TextColor=\"ChannelZone\">gap between spent and earned money</T>.</A></P>",
				[2] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">gap between spent and earned money</T> instead of <T TextColor=\"ChannelZone\">money amount</T>.</A></P>",
			},
			["Value"] =
			{
				[1] = "",
				[2] = "",
			},
		},
		
		["TechInfoPanel"] = 
		{
			["Index"] = 0,
			["NextTooltip"] =
			{
				[1] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">FPS and ping time</T> instead of <T TextColor=\"ChannelZone\">FPS and real latency</T>.</A></P>",
				[2] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">FPS and real latency</T> instead of <T TextColor=\"ChannelZone\">FPS and ping time</T>.</A></P>",
			},
			["Value"] =
			{
				[1] = "",
				[2] = "",
			},
		},
		
		["CurrencyPanel"] =
		{
			["Index"] = 0,
		},
		
		["InventoryPanel"] =
		{
			["Index"] = 0,
		},
		
		["LevelTimePanel"] =
		{
			["Index"] = 0,
			["NextTooltip"] =
			{
				["LevelGame"] =
				{
					[1] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">current level</T> instead of <T TextColor=\"ChannelZone\">time to level up</T>.</A></P>",
					[2] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">time played this session</T> instead of <T TextColor=\"ChannelZone\">current level</T>.</A></P>",
					[3] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">XP needed to level up</T> instead of <T TextColor=\"ChannelZone\">time played this session</T>.</A></P>",
					[4] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">XP gained this session</T> instead of <T TextColor=\"ChannelZone\">XP needed to level up</T>.</A></P>",
					[5] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">quests completed this session</T> instead of <T TextColor=\"ChannelZone\">XP gained this session</T>.</A></P>",
					[6] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">time to level up</T> instead of <T TextColor=\"ChannelZone\">quests completed this session</T>.</A></P>",
				},
				["ElderGame"] =
				{
					[1] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">weekly Elder Points</T> instead of <T TextColor=\"ChannelZone\">time to next gem</T>.</A></P>",
					[2] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">time played this session</T> instead of <T TextColor=\"ChannelZone\">weekly Elder Points</T>.</A></P>",
					[3] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">EP needed to next gem</T> instead of <T TextColor=\"ChannelZone\">time played this session</T>.</A></P>",
					[4] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">EP gained this session</T> instead of <T TextColor=\"ChannelZone\">EP needed to next gem</T>.</A></P>",
					[5] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">quests completed this session</T> instead of <T TextColor=\"ChannelZone\">EP gained this session</T>.</A></P>",
					[6] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">time to next gem</T> instead of <T TextColor=\"ChannelZone\">quests completed this session</T>.</A></P>",
				},
				
			},
			["Value"] =
			{
				["LevelGame"] =
				{
					[1] = "",
					[2] = "",
					[3] = "",
					[4] = "",
					[5] = "",
					[6] = "",
				},
				["ElderGame"] =
				{
					[1] = "",
					[2] = "",
					[3] = "",
					[4] = "",
					[5] = "",
					[6] = "",
				},
				
			}
		},
		
		["SocialPanel"] = 
		{
			["Index"] = 0,
			["NextTooltip"] =
			{
				[1] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">account friends and friends online</T> instead of <T TextColor=\"ChannelZone\">total friends and guild members online</T>.</A></P>",
				[2] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">total friends and guild members online</T> instead of <T TextColor=\"ChannelZone\">account friends and friends online</T>.</A></P>",
			
			},
			["Value"] =
			{
				[1] = "",
				[2] = "",
			},
			
			["Tooltip"] =
			{
				[1] = "",
				[2] = "",
			},
			
			["Size"] =
			{
				[1] =
				{
					["w"] = 185,
					["h"] = 28,
				},
				[2] =
				{
					["w"] = 195,
					["h"] = 28,
				},
			},
		},
		
		["RepairPanel"] =
		{
			["Index"] = 0,
			["NextTooltip"] =
			{
				[1] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">gear durability in %</T> instead of <T TextColor=\"ChannelZone\">repair costs</T>.</A></P>",
				[2] = "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">repair costs</T> instead of <T TextColor=\"ChannelZone\">gear durability in %</T>.</A></P>",
			},
			
			["Value"] =
			{
				[1] = "",
				[2] = "",
			},
			
			["Size"] =
			{
				[1] =
				{
					["w"] = 169,
					["h"] = 28,
				},
				[2] =
				{
					["w"] = 70,
					["h"] = 28,
				},
			},
		},
		["CraftingPanel"] =
		{
			["Index"] = 0,
			["NextTooltip"] = {}, --Dynamic
			["Value"] = {}, --Dynamic
		},
	}

	self.bHideMainWindow = false
	self.bFirstTimeConfig = true
end

-----------------------------------------------------------------------------------------------
-- NexusPanel OnDocLoaded
-----------------------------------------------------------------------------------------------
function NexusPanel:OnDocLoaded()
	self:CreateWindows()
	firstTime = true
end

function NexusPanel:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "Nexus Panel", {"ShowOptions", "", ""})
end

function NexusPanel:OnWindowManagementReady()
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMain, strName = "Nexus Panel"})
	self:SetPanelVisibility()
end

function NexusPanel:OnQuestStateChanged(queUpdated, eState)
	if eState == Quest.QuestState_Completed then
		self.data.nQuestsCompleted = self.data.nQuestsCompleted + 1
	elseif eState == Quest.QuestState_Achieved then
		self.data.nQuestsAchieved = self.data.nQuestsAchieved + 1
	end
end

function NexusPanel:CreateWindows()
	--first time
	for i in pairs(kSubPanelNames) do
		if self.config.layer[kSubPanelNames[i]] == nil or self.config.layer[kSubPanelNames[i]] == 0 then self.config.layer[kSubPanelNames[i]] = 1 end
	end
	
	if self.data.localTime == nil then self:UpdateperTick() return end
	if self.wndMain == nil or not self.wndMain then
		if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "Panel", nil, self)
		if self.wndOptionsContainer == nil or not self.wndOptionsContainer then
			self.wndOptionsContainer = Apollo.LoadForm(self.xmlDoc, "OptionsContainer", nil, self)
			--Options Container hiden
			self.wndOptionsContainer:Show(false, true)
		end
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		self.wndMain:Show(true)
		if self.config.tWindowLocation then
			self.wndMain:MoveToLocation(self.config.tWindowLocation)
		end
		
		--Show
		--self.wndMain:Show(true)
		--self.wndMain:Invoke()
		
		--if self.config.tWindowLocation and self.config.tWindowLocation ~= nil and #self.config.tWindowLocation >= 4 then
		--	testSave = self.config.tWindowLocation[1]
		--	self.wndMain:SetAnchorOffsets(self.config.tWindowLocation[1],self.config.tWindowLocation[2],self.config.tWindowLocation[3],self.config.tWindowLocation[4])
		--end
		
		
		--Subpanels
		self:CreateSubPanels()
		
		else
			self.xmlDoc = XmlDoc.CreateFromFile("NexusPanel.xml")
			self.wndMain = Apollo.LoadForm(self.xmlDoc, "Panel", nil, self)
			self.xmlDoc:RegisterCallback("OnDocLoaded", self)
		end
	end
end

function NexusPanel:CreateSubPanels()
	if self.xmlDoc ~= nil and self.wndMain then
		--
		for i,j in pairs(kSubPanelNames) do
			if self.panel and self.panel["SubPanels"] then
				--existiert auch das Subpanel?
				local nSubPanelTemp = 0
				for k,l in pairs(self.panel["SubPanels"]) do
					if l["Name"] == j then
						nSubPanelTemp = k  break
					end
				end
				
				if nSubPanelTemp > 0 and self.panel["SubPanels"][nSubPanelTemp] then
					self.panel["SubPanels"][nSubPanelTemp]["wnd"] = Apollo.LoadForm(self.xmlDoc, j,self.wndMain,self)
				else
					local subpanel =
					{
						["Name"] = j,
						["Visible"] = true,
						["Position"] = -1, --muss spter nach hinten geschoben werden
						["wnd"] = Apollo.LoadForm(self.xmlDoc, j,self.wndMain,self),
					}
					table.insert(self.panel["SubPanels"],subpanel)
				end
			else
				self.panel = 
				{
					["VisibleSubPanels"] = 0,
					["SubPanels"] = {}
				}
				
				--if self.config.panel and self.config.panel["SubPanels"] and self.config.panel["SubPanels"][kSubPanelNames[i]] then
				--	visible = self.config.panel["SubPanels"][kSubPanelNames[i]]["Visible"]
				--end
				
				local subpanel =
				{
					["Name"] = j,
					["Visible"] = true,
					["Position"] = -1,
					["wnd"] = Apollo.LoadForm(self.xmlDoc, j,self.wndMain,self),
				}
				table.insert(self.panel["SubPanels"],subpanel)
			end
			
			if kSubPanelNames[i] == "CurrencyPanel" then
				local nSubPanelTemp = 0
				for k,l in pairs(self.panel["SubPanels"]) do
					if l["Name"] == j then
						nSubPanelTemp = k  break
					end
				end
				if nSubPanelTemp > 0 and self.panel["SubPanels"][nSubPanelTemp] then
					for idx = 1, #karCurrency do
						local tData = karCurrency[idx]
						local wnd = Apollo.LoadForm(self.xmlDoc, "PickerEntry", self.panel["SubPanels"][nSubPanelTemp]["wnd"]:FindChild("BGCurrencyList"), self)
						wnd:FindChild("EntryCash"):SetMoneySystem(tData.eType)
						wnd:FindChild("EntryCash"):SetAmount(0)
						wnd:FindChild("EntryCash"):SetTooltip(tData.strDescription)
						wnd:FindChild("EntryCash"):SetData(idx)
						tData.wnd = wnd
					end
				
					self.panel["SubPanels"][nSubPanelTemp]["wnd"]:FindChild("BGCurrencyList"):ArrangeChildrenHorz(0)
				end
			end
		end
		
		self.xmlDoc = nil
		self:SortSubPanels(true)
		self:UpdateperTick()
		self:SetPanelVisibility()
	end
end

function NexusPanel:SortSubPanels(withNew)
	local sortedSubPanels = {}
	local position = 1
	
	for i in pairs(self.panel["SubPanels"]) do
		for j in pairs(self.panel["SubPanels"]) do
			if self.panel["SubPanels"][j]["Position"] == position then
				table.insert(sortedSubPanels,self.panel["SubPanels"][j])
				self.data.tLayerData[self.panel["SubPanels"][j]["Name"]]["Index"] = position
				position = position + 1
			end
		end
	end
	
	if withNew then
		for i in pairs(self.panel["SubPanels"]) do
			for j in pairs(self.panel["SubPanels"]) do
				if self.panel["SubPanels"][j]["Position"] == -1 then
					self.panel["SubPanels"][j]["Position"] = position
					table.insert(sortedSubPanels,self.panel["SubPanels"][j])
					self.data.tLayerData[self.panel["SubPanels"][j]["Name"]]["Index"] = position
					position = position + 1
				end
			end
		end
	end
	
	self.panel["SubPanels"] = nil
	self.panel["SubPanels"] = sortedSubPanels
end
--OLD
function NexusPanel:OnDocumentReady()

end
--
-----------------------------------------------------------------------------------------------
-- NexusPanel Functions
-----------------------------------------------------------------------------------------------
--only Text Updates
function NexusPanel:UpdatePanel()
	self.config.tWindowLocation = self.wndMain:GetLocation()
	if self.data.tLayerData["LocationPanel"]["Value"][1] == nil then return end --check
		if firstTime and self.panel["SubPanels"][self.data.tLayerData["MoneyPanel"]["Index"]] then --TODO!!
			self.data.moneyAtBegin = GameLib.GetPlayerCurrency():GetAmount()
			local wndClass = self.panel["SubPanels"][self.data.tLayerData["LevelTimePanel"]["Index"]]["wnd"]:FindChild("ClassBGIcon")
			if unitPlayer ~= nil then
				local nClassId = unitPlayer:GetClassId() -- von char data holen!
				if wndClass ~= nil then
					if ktClassIcon[nClassId] ~= nil then
						wndClass:SetSprite(ktClassIcon[nClassId])
					end
				end
				firstTime = false
			end
		end
	--in combat/out of combat
	unitPlayer = GameLib.GetPlayerUnit()
	if not unitPlayer then
		return
	end
	
	local bShow = false
	
	if self.config.chkbShowModeOutOfCombat ~= nil and self.config.chkbShowModeOutOfCombat then
		if not unitPlayer:IsInCombat() then bShow=true else bShow=false end
	end
	
	if not bShow then
		if self.config.chkbShowModeInCombat ~= nil and self.config.chkbShowModeInCombat then
			if unitPlayer:IsInCombat() then bShow=true else bShow=false end
		end
	end

	if not bShow then
		self.wndMain:Show(false)
		self.bHideMainWindow = true
	else
		self.wndMain:Show(true)
		self.bHideMainWindow = false
	end
	
	-- hide WS currency
	if self.config.chkbHideWSCurrency ~= nil and self.config.chkbHideWSCurrency then
		local XPBar = Apollo.GetAddon("XPBar")
		if XPBar and XPBar.wndInvokeForm then
			XPBar.wndInvokeForm:FindChild("MainCashWindow"):Show(false)
		end
	end
	if not self.bHideMainWindow then
		-- OnMouseover
		if self.config.chkbShowModeMouseOver then
			self.wndMain:SetStyle("AutoFade",true)
		else
			self.wndMain:SetStyle("AutoFade",false)
			self.wndMain:SetOpacity(1)
			self.wndMain:Show(true)
		end
		--Lock
		if self.config.chkbLockPanel then
			self.wndMain:SetStyle("Moveable",false)
		else
			self.wndMain:SetStyle("Moveable",true)
		end
		
		--SUBPANELS
		for i in pairs(kSubPanelNames) do
			if self.panel and self.panel["SubPanels"] and #self.panel["SubPanels"] > 0 and self.panel["SubPanels"][i] then
				if self.panel["SubPanels"][i]["wnd"] and self.panel["SubPanels"][i]["wnd"]:IsShown() then
					-- Location Panel
					if self.panel["SubPanels"][i]["Name"] == "LocationPanel" then
						if self.panel["SubPanels"][i]["wnd"]:FindChild("lblCurrentZone") then
							local coordstext2 = ""
							if self.data.tPos then
								self.data.strCoords = round(self.data.tPos.x).. ", ".. round(self.data.tPos.z)
								coordstext2 = round(self.data.tPos.x).. ", ".. round(self.data.tPos.z).. ", ".. round(self.data.tPos.y)
							else
								self.data.strCoords = "GPS Signal Lost!"
								coordstext2 = "GPS Signal Lost!"
							end
							
							if self.data.currentZone ~= nil then
								if self.config.subZone == nil then self.config.subZone = {} end
								if self.config.subZone.name == nil or self.config.subZone.name == "" then self.config.subZone.name = self.data.currentZone.strName end
								self.panel["SubPanels"][i]["wnd"]:FindChild("lblCurrentZone"):SetText(self.data.tLayerData["LocationPanel"]["Value"][self.config.layer["LocationPanel"]])
								self.panel["SubPanels"][i]["wnd"]:FindChild("lblCurrentZone"):SetTooltip("<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">             Zone: </A><T TextColor=\"ChannelZone\">"..self.data.currentZone.strName.. "</T></P><P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">      Subzone: </A><T TextColor=\"ChannelZone\">".. self.config.subZone.name.. "</T></P><P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Coordinates: </A><T TextColor=\"ChannelZone\">".. coordstext2.."</T></P>")
							end
						end
					end
					
					if self.panel["SubPanels"][i]["Name"] == "LevelTimePanel" then
						-- XP
						local tooltip= ""
						if self.data.nLevel < knMaxLevel then
							tooltip = "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Current Level: </A><T TextColor=\"ChannelZone\">".. tostring(self.data.nLevel).."</T></P>"
							--tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Total Time Played: </A><T TextColor=\"ChannelZone\">".. formatTime(self.data.gametime).."</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Time Played This Session: </A><T TextColor=\"ChannelZone\">".. formatTime(self.data.nTimeThisSession).."</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Current XP: </A><T TextColor=\"ChannelZone\">".. tostring(self.data.nCurrentXP.. "/".. self.data.nNeededXPTotal).." (".. tostring(round((tonumber(self.data.nCurrentXP)/tonumber(self.data.nNeededXPTotal)*100),2)) .."%)</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Rested XP: </A><T TextColor=\"ChannelZone\">".. self.data.nRestedXP.."</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Rested XP (Creatures): </A><T TextColor=\"ChannelZone\">".. self.data.nRestedXPPool.."</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">XP Needed To Level: </A><T TextColor=\"ChannelZone\">".. self.data.nNeededXP.."</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">XP Gained This Session: </A><T TextColor=\"ChannelZone\">".. self.data.totalXPGainedThisSession.."</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">XP/Hour This Session: </A><T TextColor=\"ChannelZone\">".. round(self.data.xpperHour,2).."</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Quests completed This Session: </A><T TextColor=\"ChannelZone\">".. tonumber(self.data.nQuestsCompleted).. " ("..tonumber(self.data.nTotalQuestsXPInSession).. " XP)</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Time To Level (Session Rate): </A><T TextColor=\"ChannelZone\">".. self.data.timeForLevelUp.."</T></P>"
							--tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Elder Points: </A><T TextColor=\"ChannelZone\">".. tostring(self.data.elderPoints.. "/".. self.data.elderPointsperGem).."</T></P>"
							
							self.panel["SubPanels"][i]["wnd"]:FindChild("lblTimeUntilLvlUp"):SetText(self.data.tLayerData["LevelTimePanel"]["Value"]["LevelGame"][self.config.layer["LevelTimePanel"]])
						else
							tooltip = "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Current Level: </A><T TextColor=\"ChannelZone\">".. tostring(self.data.nLevel).."</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Time Played This Session: </A><T TextColor=\"ChannelZone\">".. formatTime(self.data.nTimeThisSession).."</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Elder Points: </A><T TextColor=\"ChannelZone\">".. tostring(self.data.elderPoints.. "/".. self.data.elderPointsperGem).." (".. tostring(round((tonumber(self.data.elderPoints)/tonumber(self.data.elderPointsperGem)*100),2)) .."%)</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Rested EP: </A><T TextColor=\"ChannelZone\">".. self.data.nRestedXP.."</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Rested EP (Creatures): </A><T TextColor=\"ChannelZone\">".. self.data.nRestedXPPool.."</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">EP Gained This Session: </A><T TextColor=\"ChannelZone\">".. self.data.totalElderPointsGainedThisSession.."</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">EP/Hour This Session: </A><T TextColor=\"ChannelZone\">".. round(self.data.elderPointsperHour,2).."</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Weekly EP: </A><T TextColor=\"ChannelZone\">".. tostring(self.data.strWeeklyElderPoints .. "/".. self.data.weeklyElderPointsCap).." (".. tostring(round((tonumber(self.data.weeklyElderPoints)/tonumber(self.data.weeklyElderPointsCap)*100),2)) .."%)</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Quests completed This Session: </A><T TextColor=\"ChannelZone\">".. tonumber(self.data.nQuestsCompleted).. "</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Time To Next Gem (Session Rate): </A><T TextColor=\"ChannelZone\">".. self.data.timeForNextGem.."</T></P>"
								
							self.panel["SubPanels"][i]["wnd"]:FindChild("lblTimeUntilLvlUp"):SetText(self.data.tLayerData["LevelTimePanel"]["Value"]["ElderGame"][self.config.layer["LevelTimePanel"]])
						end
						self.panel["SubPanels"][i]["wnd"]:FindChild("lblTimeUntilLvlUp"):SetTooltip(tooltip)
					end
					
					--MONEY
					if self.panel["SubPanels"][i]["Name"] == "MoneyPanel" then
						local text = ""
						local tooltip = ""
						local copper = self.data.totalMoneyLootedThisSession
						
						self.panel["SubPanels"][i]["wnd"]:FindChild("CashWindow"):SetAmount(self.data.tLayerData["MoneyPanel"]["Value"][self.config.layer["MoneyPanel"]])
						--vorerst, spter auch Tooltips ber layer
						if self.config.layer["MoneyPanel"] == 1 then
							if copper < 0 then self.panel["SubPanels"][i]["wnd"]:FindChild("CashWindow"):SetTooltip("<P Font=\"CRB_Button\"><T TextColor=\"ChannelZone\">Money <T TextColor=\"AddonError\">Loss</T> In Session</T></P>") elseif copper > 0 then self.panel["SubPanels"][i]["wnd"]:FindChild("CashWindow"):SetTooltip("<P Font=\"CRB_Button\"><T TextColor=\"ChannelZone\">Money <T TextColor=\"AddonOk\">Profit</T> In Session</T></P>") else self.panel["SubPanels"][i]["wnd"]:FindChild("CashWindow"):SetTooltip("Nothing earned or spent this session") end
							if copper < 0 then
								self.panel["SubPanels"][i]["wnd"]:FindChild("CashWindow"):SetTextColor(ApolloColor.new("AddonError"))
							elseif copper >= 0 then
								self.panel["SubPanels"][i]["wnd"]:FindChild("CashWindow"):SetTextColor(ApolloColor.new("AddonOk"))
							end
						else
							self.panel["SubPanels"][i]["wnd"]:FindChild("CashWindow"):SetTooltip("<P Font=\"CRB_Button\"><T TextColor=\"ChannelZone\">Money amount</T></P>")
							self.panel["SubPanels"][i]["wnd"]:FindChild("CashWindow"):SetTextColor("ffffffff")
						end
					end
					
					--Repair
					if self.panel["SubPanels"][i]["Name"] == "RepairPanel" and #self.data.tLayerData["RepairPanel"]["Value"] > 1 then
						local strColor = ""
						local strTooltip=""
						local bRed = false
						for i,j in pairs(self.data.tRepairItems) do
								if j["Durability"] == 100 then strColor = "AddonOk"
								elseif j["Durability"] > 0 then strColor = "AttributeName"
								else strColor = "AddonError" bRed=true
								end
								strTooltip = strTooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">".. j["SlotName"].. ": ".. "</A><T TextColor=\"".. strColor.. "\">".. j["Durability"].. "%</T></P>"
						end
						if self.config.layer["RepairPanel"] == 1 then
							if self.panel["SubPanels"][i]["wnd"]:FindChild("CashWindow") then
								self.panel["SubPanels"][i]["wnd"]:FindChild("CashWindow"):SetAmount(self.data.tLayerData["RepairPanel"]["Value"][self.config.layer["RepairPanel"]],false)
								self.panel["SubPanels"][i]["wnd"]:FindChild("CashWindow"):SetTooltip(strTooltip)
								self.panel["SubPanels"][i]["wnd"]:FindChild("lblRepairPercent"):Show(false)
								self.panel["SubPanels"][i]["wnd"]:FindChild("CashWindow"):Show(true)
							end
						else
							self.panel["SubPanels"][i]["wnd"]:FindChild("lblRepairPercent"):SetText(self.data.tLayerData["RepairPanel"]["Value"][self.config.layer["RepairPanel"]].. "%")
							
							if bRed then
								strColor = "AddonError"
							elseif self.data.actRepairPercent == 100 then
								strColor = "AddonOk"
							else
								strColor = "AttributeName"
							end
							
							self.panel["SubPanels"][i]["wnd"]:FindChild("lblRepairPercent"):SetTextColor(ApolloColor.new(strColor))
							self.panel["SubPanels"][i]["wnd"]:FindChild("lblRepairPercent"):SetTooltip(strTooltip)
							self.panel["SubPanels"][i]["wnd"]:FindChild("CashWindow"):Show(false)
							self.panel["SubPanels"][i]["wnd"]:FindChild("lblRepairPercent"):Show(true)
						end
					
					end
					
					--FPS / Technical Info
					if self.panel["SubPanels"][i]["Name"] == "TechInfoPanel" then
						local actFPS = round(tonumber(self.data.actFPS))
						self.panel["SubPanels"][i]["wnd"]:FindChild("lblFPSInfo"):SetText(tostring(actFPS))
				
						if actFPS < 15 then
							self.panel["SubPanels"][i]["wnd"]:FindChild("lblFPSInfo"):SetTextColor(ApolloColor.new("AddonError"))
						elseif actFPS < 30 then
							self.panel["SubPanels"][i]["wnd"]:FindChild("lblFPSInfo"):SetTextColor(ApolloColor.new("AttributeName"))
						else
							self.panel["SubPanels"][i]["wnd"]:FindChild("lblFPSInfo"):SetTextColor(ApolloColor.new("AddonOk"))
						end
						
						--LATENCY
						self.panel["SubPanels"][i]["wnd"]:FindChild("lblLatencyInfo"):SetText(self.data.tLayerData["TechInfoPanel"]["Value"][self.config.layer["TechInfoPanel"]])
						
						local tooltip = "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">            Current FPS: </A><T TextColor=\"ChannelZone\">".. tostring(actFPS).. "</T></P>"
						tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">          Average FPS: </A><T TextColor=\"ChannelZone\">".. tostring(round(tonumber(self.data.fpsPerSec),1)).. "</T></P>"
						--tooltip = tooltip.. ""
						tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">      Current latency: </A><T TextColor=\"ChannelZone\">".. tostring(round(tonumber(self.data.actLatency))).. " ms</T></P>"
						tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">    Average latency: </A><T TextColor=\"ChannelZone\">".. tostring(round(tonumber(self.data.averageLatency))).. " ms</T></P>"
						tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">  Current ping time: </A><T TextColor=\"ChannelZone\">".. tostring(round(tonumber(self.data.actPingTime))).. " ms</T></P>"
						tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Average ping time: </A><T TextColor=\"ChannelZone\">".. tostring(round(tonumber(self.data.averagePingTime))).. " ms</T></P>"
						self.panel["SubPanels"][i]["wnd"]:FindChild("BGTech"):SetTooltip(tooltip)
					end
					
					--Time
					if self.panel["SubPanels"][i]["Name"] == "TimePanel" then
						if self.panel["SubPanels"][i]["wnd"]:FindChild("lblTimeInfo") then
							--self.wndTimePanel:FindChild("lblTimeInfo"):SetText(string.format("%02d:%02d", tostring(self.data.localTime.nHour), tostring(self.data.localTime.nMinute)))
							self.panel["SubPanels"][i]["wnd"]:FindChild("lblTimeInfo"):SetText(self.data.tLayerData["TimePanel"]["Value"][self.config.layer["TimePanel"]])
									
							local tooltip = "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">  Local Time: </A><T TextColor=\"ChannelZone\">"..string.format("%02d:%02d:%02d",tostring(self.data.localTime.nHour),tostring(self.data.localTime.nMinute),tostring(self.data.localTime.nSecond)).."</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Server Time: </A><T TextColor=\"ChannelZone\">"..string.format("%02d:%02d:%02d",tostring(self.data.serverTime.nHour),tostring(self.data.serverTime.nMinute),tostring(self.data.serverTime.nSecond)).."</T></P>"
							tooltip = tooltip.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\"> Game Time: </A><T TextColor=\"ChannelZone\">"..tostring(formatTime(self.data.gameTime)) .."</T></P>"
					
							self.panel["SubPanels"][i]["wnd"]:FindChild("lblTimeInfo"):SetTooltip(tooltip)
						end
					end
					
					--Social
					if self.panel["SubPanels"][i]["Name"] == "SocialPanel" then
						self.panel["SubPanels"][i]["wnd"]:FindChild("lblOnlineInfo"):SetText(self.data.tLayerData["SocialPanel"]["Value"][self.config.layer["SocialPanel"]])
						self.panel["SubPanels"][i]["wnd"]:FindChild("lblOnlineInfo"):SetTooltip(self.data.tLayerData["SocialPanel"]["Tooltip"][self.config.layer["SocialPanel"]])
						self.panel["SubPanels"][i]["wnd"]:FindChild("btnPost"):FindChild("lblMailsCount"):SetText(tostring(self.data.nUnreadMessages))
						if self.data.nUnreadMessages > 0 then
							self.panel["SubPanels"][i]["wnd"]:FindChild("btnPost"):SetTextColor(ApolloColor.new("AddonOk"))
						else
							self.panel["SubPanels"][i]["wnd"]:FindChild("btnPost"):SetTextColor(ApolloColor.new("CRB_Button"))
						end
						self.panel["SubPanels"][i]["wnd"]:FindChild("btnPost"):FindChild("lblMailsCount"):SetTooltip("<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Unread messages: </A><T TextColor=\"ChannelZone\">".. self.data.nUnreadMessages.. "</T></P>")
					end
					
					--Currency 
					if self.panel["SubPanels"][i]["Name"] == "CurrencyPanel" then
						for key, wndCurr in pairs(self.panel["SubPanels"][i]["wnd"]:FindChild("BGCurrencyList"):GetChildren()) do
							local tData = karCurrency[wndCurr:FindChild("EntryCash"):GetData()]
							tData.wnd:FindChild("EntryCash"):SetAmount(GameLib.GetPlayerCurrency(tData.eType):GetAmount())
						end
					end
					
					--Inventory
					if self.panel["SubPanels"][i]["Name"] == "InventoryPanel" then
						self.panel["SubPanels"][i]["wnd"]:FindChild("lblBagSlots"):SetText(self.data.nUsedInventorySlots.. "/".. self.data.nTotalInventorySlots)
						
						local bagPerc = self.data.nEmptyInventorySlots / self.data.nTotalInventorySlots * 100
						local color = ""
						if bagPerc > 60 then
							color = "AddonOk"
						elseif bagPerc > 20 then
							color = "AttributeName"
						else
							color = "AddonError"
						end
						self.panel["SubPanels"][i]["wnd"]:FindChild("lblBagSlots"):SetTextColor(ApolloColor.new(color))
						self.panel["SubPanels"][i]["wnd"]:FindChild("lblBagSlots"):SetTooltip("<P Font=\"CRB_Button\"><A TextColor=\"".. color.. "\">".. self.data.nEmptyInventorySlots.." free bag slots</A></P>")
					end
					
					--CraftingXP
					if self.panel["SubPanels"][i]["Name"] == "CraftingPanel" then
						if self.panel["SubPanels"][i]["wnd"]:FindChild("CraftingXP") and self.data.tLayerData["CraftingPanel"] and self.data.tLayerData["CraftingPanel"]["Value"] and self.config.layer["CraftingPanel"] and self.data.tLayerData["CraftingPanel"]["Value"][self.config.layer["CraftingPanel"]] and self.data.tLayerData["CraftingPanel"]["Value"][self.config.layer["CraftingPanel"]]["nXpForNextTier"] then
							self.panel["SubPanels"][i]["wnd"]:FindChild("CraftingXP"):SetMax(self.data.tLayerData["CraftingPanel"]["Value"][self.config.layer["CraftingPanel"]]["nXpForNextTier"])
							self.panel["SubPanels"][i]["wnd"]:FindChild("CraftingXP"):SetProgress(self.data.tLayerData["CraftingPanel"]["Value"][self.config.layer["CraftingPanel"]]["nXp"])
							self.panel["SubPanels"][i]["wnd"]:FindChild("CraftingXP"):SetTooltip(self.data.tLayerData["CraftingPanel"]["Tooltip"][self.config.layer["CraftingPanel"]])
						end
					end
				end		
			end
		end	
	end
end


function NexusPanel:UpdateLayer()
	for i,j in pairs(self.panel["SubPanels"]) do
		if self.panel and self.panel["SubPanels"] and #self.panel["SubPanels"] > 0 and self.panel["SubPanels"][i] then
			if self.data.tLayerData and self.data.tLayerData[j["Name"]] and self.data.tLayerData[j["Name"]]["NextTooltip"] and self.config.layer[j["Name"]] then
				if self.data.tLayerData[j["Name"]]["NextTooltip"]["LevelGame"] then
					if self.data.nLevel and self.data.nLevel == knMaxLevel then
						self.panel["SubPanels"][i]["wnd"]:FindChild("btnNext"):SetTooltip(tostring(self.data.tLayerData[j["Name"]]["NextTooltip"]["ElderGame"][self.config.layer[j["Name"]]]))
					elseif self.data.nLevel and self.data.nLevel < knMaxLevel then
						self.panel["SubPanels"][i]["wnd"]:FindChild("btnNext"):SetTooltip(tostring(self.data.tLayerData[j["Name"]]["NextTooltip"]["LevelGame"][self.config.layer[j["Name"]]]))
					end
				else
					self.panel["SubPanels"][i]["wnd"]:FindChild("btnNext"):SetTooltip(tostring(self.data.tLayerData[j["Name"]]["NextTooltip"][self.config.layer[j["Name"]]]))
				end
			end
		end
	end

	self:UpdatePanel()
end

--Hier dann noch die Gre der Subpanel je nach layer
function NexusPanel:SetPanelVisibility()
if self.data.tLayerData["TimePanel"]["Size"][self.config.layer["TimePanel"]] == nil then self:UpdateperTick() return end --check
	if self.wndMain ~= nil and self.wndMain then
		--BG
		if self.config.chkbBGTransparency then
			self.wndMain:FindChild("BGStyle1"):Show(false)
		else
			self.wndMain:FindChild("BGStyle1"):Show(true)
		end
		local widthTotal=0
		--Subpanels
		if self.panel and self.panel["SubPanels"] and #self.panel["SubPanels"] > 0 then
			for i in pairs(self.panel["SubPanels"]) do
				if self.panel["SubPanels"][i]["Visible"] then
					local l,t,r,b = 0,1,0,29
					if self.data.tLayerData[self.panel["SubPanels"][i]["Name"]] and self.data.tLayerData[self.panel["SubPanels"][i]["Name"]]["Size"] and self.data.tLayerData[self.panel["SubPanels"][i]["Name"]]["Size"][self.config.layer[self.panel["SubPanels"][i]["Name"]]] then
						r = self.data.tLayerData[self.panel["SubPanels"][i]["Name"]]["Size"][self.config.layer[self.panel["SubPanels"][i]["Name"]]]["w"]
					else
						l,_,r,_ = self.panel["SubPanels"][i]["wnd"]:GetAnchorOffsets()
					end
					self.panel["SubPanels"][i]["wnd"]:SetAnchorOffsets(widthTotal+1,t,(r-l)+widthTotal+1,b)
					widthTotal = widthTotal + 1 + (r-l)
					if self.config.chkbBordersTransparency then
						self.panel["SubPanels"][i]["wnd"]:SetSprite("")
					else
						self.panel["SubPanels"][i]["wnd"]:SetSprite("AbilitiesSprites:spr_StatVertProgBase")
					end
					self.panel["SubPanels"][i]["wnd"]:Show(true)
				else
					self.panel["SubPanels"][i]["wnd"]:Show(false)
				end
			end
		end
		
				
		--am Ende noch fr Right +1 in widthTotal
		if widthTotal <= 0 then
			if self.config.chkbShowModeMouseOver then
				--self.wndMain:Show(false)
				--self.wndMain:SetOpacity(0)
			else
				self.wndMain:Show(false)
				self.bHideMainWindow = true
			end
		else
			widthTotal = widthTotal + 1
			local l,t,r,b = self.wndMain:GetAnchorOffsets()
			self.wndMain:SetAnchorOffsets(l,t,widthTotal+l,b)
			if self.config.chkbShowModeMouseOver then
				--self.wndMain:Show(false)
				--self.wndMain:SetOpacity(0)
			else
				--self.wndMain:SetOpacity(1)
				--self.wndMain:Show(false)
				self.bHideMainWindow = false
			end	
		end
		
		if self.wndOptionsContainer and self.wndOptionsContainer:FindChild("btnSetPositionMode") and self.wndOptionsContainer:FindChild("btnSetPositionMode"):GetText() == "Lock positions" then
			self:SetPositionMode(false)
			self:SetPositionMode(true)
		end
		--Update Tracked WIndow
		local OptInterface = Apollo.GetAddon("OptionsInterface")
		local addon = Apollo.GetAddonInfo("NexusPanel")
		--for x,y in pairs(addon) do
		--end	
		if OptInterface then
			OptInterface:UpdateTrackedWindow(self.wndMain)
		end
		
		self:UpdateLayer()
	end
end

function NexusPanel:OnSetPositionMode(wndHandler,wndControl)
	if wndControl then
		if wndControl:GetText() == "Edit positions" then
			wndControl:SetText("Lock positions")
			self:SetPositionMode(true)
		else
			wndControl:SetText("Edit positions")
			self:SetPositionMode(false)
		end
	end
end

function NexusPanel:SetPositionMode(on)
	--fr mode=on erst anzahl visible SubPanels zhlen
	if self.panel and self.panel["SubPanels"] then
		local nVisibleSubPanels = 0
		local first = true
		if on then
			for i,j in pairs(self.panel["SubPanels"]) do
				if j["Visible"] then nVisibleSubPanels = nVisibleSubPanels + 1 end
			end
		end

		for i,j in pairs(self.panel["SubPanels"]) do
			if self.panel["SubPanels"][i]["wnd"] then
				if self.panel["SubPanels"][i]["wnd"]:IsShown() then
					for k,l in pairs(self.panel["SubPanels"][i]["wnd"]:GetChildren()) do
						if l:GetName() == "BGMove" then
							l:Show(on)
							
							l:FindChild("btnArrowLeft"):Show(true)
							l:FindChild("btnArrowRight"):Show(true)
							
							if first and j["Visible"] then
								l:FindChild("btnArrowLeft"):Show(false)
								first = false
							elseif i == nVisibleSubPanels then
								l:FindChild("btnArrowRight"):Show(false)
							end
						else
							if on then l:SetOpacity(0.5) else l:SetOpacity(1) end
							
							if l:GetName() == "btnNext" or l:GetName() == "btnPost" then
								if on then l:Show(false) else l:Show(true) end
							end
						end
					end
				else
					nVisibleSubPanels = nVisibleSubPanels + 1
				end
			end
		end
	end
end

function NexusPanel:SetTechData()
	self.data.totalFPS = self.data.totalFPS + GameLib.GetFrameRate()
	self.data.totalLatency = self.data.totalLatency + GameLib.GetLatency()
	self.data.totalPingTime = self.data.totalPingTime + GameLib.GetPingTime()
	
	self.data.actFPS = GameLib.GetFrameRate()
	self.data.fpsPerSec = (self.data.totalFPS / self.data.totalTicks)	
	
	self.data.actLatency = GameLib.GetLatency()
	self.data.averageLatency = (self.data.totalLatency / self.data.totalTicks)
	
	self.data.actPingTime = GameLib.GetPingTime()
	self.data.averagePingTime = (self.data.totalPingTime / self.data.totalTicks)
	
	--erst einmal nur Ping, spter TODO layer auf verschiedene labels
	self.data.tLayerData["TechInfoPanel"]["Value"] = 
	{
		[1] = tostring(round(self.data.actLatency)).. " ms",
		[2] = tostring(round(self.data.actPingTime)).. " ms",
	}
end

function NexusPanel:OnSubZoneChanged(id, name)
	subZone = {}
	subZone.id = id
	subZone.name = name
	self.config.subZone = subZone
	if self.wndMain ~= nil then
		self:UpdatePanel()
	end
end

-- old
function NexusPanel:OnLootedMoney(monLooted)
	
end

--evtl. per Tick von Char Info holen?
function NexusPanel:OnExperienceGained(eReason, unitTarget, strText, fDelay, nAmount)
	self.data.totalXPGainedThisSession = self.data.totalXPGainedThisSession + nAmount
	if eReason and eReason == 3 then
		self.data.nTotalQuestsXPInSession = self.data.nTotalQuestsXPInSession + nAmount
	end
end

function NexusPanel:OnElderPointsGained(nAmount)
	self.data.totalElderPointsGainedThisSession = self.data.totalElderPointsGainedThisSession + nAmount
end

function NexusPanel:UpdateperTick()
	--ZONE
	unit = GameLib.GetPlayerUnit()
	if unit then
		self.data.tPos = unit:GetPosition()
		self.data.nLevel = unit:GetBasicStats().nLevel
	end

	worldId = GameLib.GetCurrentWorldId()
	self.data.currentZone = GameLib.GetCurrentZoneMap()
	
	if self.data.currentZone == nil then return end
	
	self.data.tLayerData["LocationPanel"]["Value"] = 
	{
		[1] = self.config.subZone.name.. " (".. self.data.strCoords.. ")",
		[2] = self.data.currentZone.strName.. " (".. self.data.strCoords.. ")",
		[3] = self.config.subZone.name,
		[4] = self.data.currentZone.strName,
	}
	
	--Elder Points
	self.data.elderPoints =	tonumber(GetElderPoints())
	self.data.elderPointsperGem = tonumber(GameLib.ElderPointsPerGem)
	self.data.nNeededElderPoints = tonumber(self.data.elderPointsperGem - self.data.elderPoints)
	self.data.weeklyElderPoints = tonumber(GetPeriodicElderPoints())
	
	self.data.elderPointsPerTick = tonumber(self.data.totalElderPointsGainedThisSession / self.data.totalTicks)
	
	self.data.elderPointsperHour = tonumber(self.data.elderPointsPerTick * 3600)

	if self.data.elderPointsperHour > 0 then
		self.data.timeForNextGem = tonumber((self.data.nNeededElderPoints / self.data.elderPointsperHour) * 3600)
	else
		self.data.timeForNextGem = 0
	end
	
	--XP
	self.data.nCurrentXP = tonumber(GetXp() - GetXpToCurrentLevel())
	self.data.nNeededXPTotal = tonumber(GetXpToNextLevel())			
	self.data.nNeededXP = tonumber(self.data.nNeededXPTotal - self.data.nCurrentXP)
	self.data.nRestedXP = tonumber(GetRestXp())							
	self.data.nRestedXPPool = tonumber(GetRestXpKillCreaturePool())
	self.data.xpPerTick = tonumber(self.data.totalXPGainedThisSession / self.data.totalTicks)
	--XP pro Minute umrechnen
	self.data.xpperHour = tonumber(self.data.xpPerTick * 3600)
	if self.data.xpperHour > 0 then
		self.data.timeForLevelUp = tonumber((self.data.nNeededXP / self.data.xpperHour) * 3600)
	else
		self.data.timeForLevelUp = 0
	end
	
	if self.data.timeForLevelUp == nil or self.data.timeForLevelUp == 0 then self.data.timeForLevelUp = tostring("Infinite") else self.data.timeForLevelUp = tostring(formatTime(self.data.timeForLevelUp)) end
	if self.data.timeForNextGem == nil or self.data.timeForNextGem == 0 then self.data.timeForNextGem = tostring("Infinite") else self.data.timeForNextGem = tostring(formatTime(self.data.timeForNextGem)) end
						
	if self.data.nNeededXP > 999999 then self.data.nNeededXP = round((self.data.nNeededXP / 1000)).. "k" end
	if self.data.weeklyElderPoints > 999999 then self.data.strWeeklyElderPoints = tostring(round((self.data.weeklyElderPoints / 1000)).. "k") else self.data.strWeeklyElderPoints = tostring(self.data.weeklyElderPoints) end
	if self.data.nNeededElderPoints > 999999 then self.data.nNeededElderPoints = round((self.data.nNeededElderPoints / 1000)).. "k" end
	
	self.data.tLayerData["LevelTimePanel"]["Value"] = 
	{
		["LevelGame"] =
		{
			[1] = self.data.timeForLevelUp,
			[2] = "Level ".. self.data.nLevel,
			[3] = formatTime(self.data.nTimeThisSession),
			[4] = self.data.nNeededXP.. " (".. tostring(round((tonumber(self.data.nCurrentXP)/tonumber(self.data.nNeededXPTotal)*100),2)) .."%)",
			[5] = self.data.totalXPGainedThisSession.. " XP",
			[6] = "Quests: ".. self.data.nQuestsCompleted,
		},
		["ElderGame"] =
		{
			[1] = self.data.timeForNextGem,
			[2] = self.data.strWeeklyElderPoints.. " (".. tostring(round((tonumber(self.data.weeklyElderPoints)/tonumber(self.data.weeklyElderPointsCap)*100),2)) .."%)",
			[3] = formatTime(self.data.nTimeThisSession),
			[4] = self.data.nNeededElderPoints.. " (".. tostring(round((tonumber(self.data.elderPoints)/tonumber(self.data.elderPointsperGem)*100),2)) .."%)",
			[5] = self.data.totalElderPointsGainedThisSession.. " EP",
			[6] = "Quests: ".. self.data.nQuestsCompleted,
		}
	}
	
	--Gametime
	self.data.gametime = GameLib.GetGameTime()
	self.data.nTimeThisSession = time() - self.data.nSessionStart
	if self.data.nLevelStart then
		self.data.nTimeThisLevel = self.data.nTotalTimePlayed - self.data.nLevelStart
	end
	self.data.nTotalTimePlayed = self.data.nSecondsPlayed + self.data.nTimeThisSession
	
	--Repair
	self.data.repairCosts = GameLib.GetRepairAllCost()
	
	self.data.tRepairItems = {}
	local tInventoryItems = nil
	local nTotalPercent = 0
	local nItemCounter = 0
	if unit then
		tInventoryItems = unit:GetEquippedItems()
		for i in pairs(tInventoryItems) do
			if tInventoryItems[i]:GetMaxDurability() > 0 then
				local percent = round(tInventoryItems[i]:GetDurability() / tInventoryItems[i]:GetMaxDurability() * 100,0)
				if percent < 100 then
					local tRepairItem =
					{
						["ItemName"] = tInventoryItems[i]:GetName(),
						["SlotName"] = tInventoryItems[i]:GetSlotName(),
						["Durability"] = percent,
						["RepairCosts"] = tInventoryItems[i]:GetRepairCost(),
					}
					table.insert(self.data.tRepairItems,tRepairItem)
					nTotalPercent = nTotalPercent + percent
					
					nItemCounter = nItemCounter + 1
				end
			end
		end
		
		--Sortieren		
		local tRepairItemsTemp = {}
		if #self.data.tRepairItems > 0 then
			for i,j in pairs(self.data.tRepairItems) do
				if #tRepairItemsTemp > 0 then
					if j["Durability"] >= tRepairItemsTemp[table.getn(tRepairItemsTemp)]["Durability"] then
						table.insert(tRepairItemsTemp,j)
					else
						for k,l in pairs(tRepairItemsTemp) do
							if j["Durability"] <= l["Durability"] then
								table.insert(tRepairItemsTemp,k,j)
								break
							end
						end
					end
				else
					table.insert(tRepairItemsTemp,j)
				end
			end
		end
		
		self.data.tRepairItems = nil
		self.data.tRepairItems = tRepairItemsTemp
		
		if nItemCounter > 0 then
			self.data.actRepairPercent = round(nTotalPercent / nItemCounter,0)
		else
			self.data.actRepairPercent = 100
		end
	end
	
	self.data.tLayerData["RepairPanel"]["Value"] = 
	{
		[1] = self.data.repairCosts,
		[2] = self.data.actRepairPercent,
	}
	--Money
	self.data.totalMoneyLootedThisSession = GameLib.GetPlayerCurrency():GetAmount() - self.data.moneyAtBegin
	
	self.data.tLayerData["MoneyPanel"]["Value"] = 
	{
		[1] = math.abs(self.data.totalMoneyLootedThisSession),
		[2] = GameLib.GetPlayerCurrency(),
	}
	
	--FPS/Ping/Latency
	self:SetTechData()

	--Time
	self.data.localTime = GameLib:GetLocalTime()
	self.data.gameTime = GameLib:GetGameTime()
	self.data.serverTime = GameLib:GetServerTime()
	
	self.data.tLayerData["TimePanel"]["Value"] =
	{
		[1] = os.date("%H:%M",os.time{year=self.data.localTime.nYear, month=self.data.localTime.nMonth, day=self.data.localTime.nDay, hour=self.data.localTime.nHour, min=self.data.localTime.nMinute, sec=self.data.localTime.nSec}),
		[2] = os.date("%H:%M",os.time{year=self.data.serverTime.nYear, month=self.data.serverTime.nMonth, day=self.data.serverTime.nDay, hour=self.data.serverTime.nHour, min=self.data.serverTime.nMinute, sec=self.data.serverTime.nSec}),
		[3] = os.date("%I:%M %p",os.time{year=self.data.localTime.nYear, month=self.data.localTime.nMonth, day=self.data.localTime.nDay, hour=self.data.localTime.nHour, min=self.data.localTime.nMinute, sec=self.data.localTime.nSec}),
		[4] = os.date("%I:%M %p",os.time{year=self.data.serverTime.nYear, month=self.data.serverTime.nMonth, day=self.data.serverTime.nDay, hour=self.data.serverTime.nHour, min=self.data.serverTime.nMinute, sec=self.data.serverTime.nSec}),
	}
	
	--SocialPanel
	self.data.onlineAccountFriends = {}
	self.data.onlineFriends = {}
	self.data.nOnlineFriends = 0
	self.data.nOnlineAccountFriends = 0
	
	for key, tFriend in pairs(FriendshipLib.GetList()) do
		if tFriend.fLastOnline == 0 then
			self.data.nOnlineFriends = self.data.nOnlineFriends + 1
			table.insert(self.data.onlineFriends, tFriend.strCharacterName)
		end
	end	
			
	for key, tAccFriend in pairs(FriendshipLib.GetAccountList()) do
		if tAccFriend.arCharacters then 
			self.data.nOnlineAccountFriends = self.data.nOnlineAccountFriends + 1
			table.insert(self.data.onlineAccountFriends, tAccFriend.strCharacterName)
		end
	end

	local nCircles = 0
	self.data.tGuild = nil
	self.data.nOnlineGuildMembers = 0
	self.data.circles = {}
	
	for idx, guild in pairs(GuildLib.GetGuilds()) do
		if guild:GetType() == GuildLib.GuildType_Guild then
			self.data.tGuild = guild
			self.data.nOnlineGuildMembers = guild:GetOnlineMemberCount()	
		elseif guild:GetType() == GuildLib.GuildType_Circle then
			nCircles = nCircles + 1
			self.data.circles[nCircles] = { name=guild:GetName(), count=guild:GetOnlineMemberCount() } 				
		end
	end	
	
	if self.data.tGuild ~= nil then
		self.data.nOnlineGuildMembers = self.data.nOnlineGuildMembers - 1 --myself
	end
	
	for circle in pairs(self.data.circles) do
		self.data.circles[circle].count = self.data.circles[circle].count - 1 --myself
	end
	
	--
	
	self.data.nUnreadMessages = 0
	for idx, tMessage in pairs(MailSystemLib.GetInbox()) do
		local tMessageInfo = tMessage:GetMessageInfo()
		if tMessageInfo and not tMessageInfo.bIsRead then
			self.data.nUnreadMessages = self.data.nUnreadMessages + 1
		end
	end	
		
	self.data.nOnlineFriendsTotal = self.data.nOnlineFriends + self.data.nOnlineAccountFriends --Total
	
	-- Value
	strOnlineGuildMembers = ""
	if self.data.tGuild == nil then strOnlineGuildMembers = "-" else strOnlineGuildMembers = tostring(self.data.nOnlineGuildMembers) end
	self.data.tLayerData["SocialPanel"]["Value"] =
	{
		[1] = "Friends: ".. self.data.nOnlineFriendsTotal.. " Guild: "..strOnlineGuildMembers,
		[2] = "Acc. Friends: ".. self.data.nOnlineAccountFriends.. " Friends: "..self.data.nOnlineFriends,
	}
	
	--Tooltip
	local strCircles = ""
	if #self.data.circles > 0 then
		for circle in pairs(self.data.circles) do
			strCircles = strCircles.. "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">- ".. self.data.circles[circle].name.. ": </A><T TextColor=\"ChannelZone\">".. self.data.circles[circle].count.. "</T></P>"
		end
	end
	
	local strFriendNames= ""
	if #self.data.onlineAccountFriends > 0 then
		for friend in pairs(self.data.onlineAccountFriends) do
			strFriendNames = strFriendNames.. "<P Font=\"CRB_Button\"><T TextColor=\"ChannelZone\">- ".. self.data.onlineAccountFriends[friend].. "</T></P>"
		end
	end
	
	if #self.data.onlineFriends > 0 then
		for friend in pairs(self.data.onlineFriends) do
			strFriendNames = strFriendNames.. "<P Font=\"CRB_Button\"><T TextColor=\"ChannelZone\">- ".. self.data.onlineFriends[friend].. "</T></P>"
		end
	end
	
	local strTooltipFriends = ""
	local strTooltipFriendsName = ""
	local strTooltipGuild = ""
	local strTooltipCircles = ""
	
	strTooltipFriends = "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Total friends online: </A><T TextColor=\"ChannelZone\">".. self.data.nOnlineFriendsTotal.. "</T></P><P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Account friends online: </A><T TextColor=\"ChannelZone\">".. self.data.nOnlineAccountFriends.."</T></P><P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Friends online: </A><T TextColor=\"ChannelZone\">".. self.data.nOnlineFriends.. "</T></P>"
	if #self.data.onlineAccountFriends > 0 or #self.data.onlineFriends > 0 then strTooltipFriendsName = "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">All friends online: </A></P>".. strFriendNames end
	if self.data.tGuild ~= nil then strTooltipGuild = "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Other guild members online: </A><T TextColor=\"ChannelZone\">"..self.data.nOnlineGuildMembers.. "</T></P>" end
	if #self.data.circles > 0 then strTooltipCircles  = "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">Other circle members online:</A></P>".. strCircles end
	
	self.data.tLayerData["SocialPanel"]["Tooltip"] =
	{
		[1] = strTooltipFriends.. strTooltipGuild.. strTooltipCircles.. strTooltipFriendsName,
		[2] = strTooltipFriends.. strTooltipGuild.. strTooltipCircles.. strTooltipFriendsName,
	}
	
	--Bag Panel
	self.data.nTotalInventorySlots = GameLib.GetTotalInventorySlots()
	self.data.nEmptyInventorySlots = GameLib.GetEmptyInventorySlots()
	self.data.nUsedInventorySlots = self.data.nTotalInventorySlots - self.data.nEmptyInventorySlots
	
	--Quests
	if self.data.nQuestsAchieved == nil then self.data.nQuestsAchieved = 0 end
	if self.data.nQuestsCompleted == nil then self.data.nQuestsCompleted = 0 end
	if self.data.nTotalQuestsXPInSession == nil then self.data.nTotalQuestsXPInSession = 0 end
	
	-- Arena / BG
	--self:WriteChat(tostring(MatchingGame.GetPvpRating(MatchingGame.RatingType.RatedBattleground).nRating))
	--GameLib.GetGearScore() or 0
	
	--Crafting Panel
	self.data.craftingXP = {}
	
	for idx, tCurrTradeskill in ipairs(CraftingLib.GetKnownTradeskills()) do
		local tInfo = CraftingLib.GetTradeskillInfo(tCurrTradeskill.eId)
		if tInfo then
			self.data.craftingXP[idx] = {}
			self.data.craftingXP[idx]["eId"] = tCurrTradeskill.eId
			self.data.craftingXP[idx]["strName"] = tInfo.strName
			self.data.craftingXP[idx]["bIsHobby"] = tInfo.bIsHobby
			self.data.craftingXP[idx]["bIsActive"] = tInfo.bIsActive
			self.data.craftingXP[idx]["nXp"] = tInfo.nXp
			self.data.craftingXP[idx]["nXpForNextTier"] = tInfo.nXpForNextTier
			
			local tMiddleCategories = AchievementsLib.GetTradeskillAchievementCategoryTree(tCurrTradeskill.eId)
			if tMiddleCategories and tMiddleCategories.tSubGroups then
				self.data.craftingXP[idx]["subGroupName"] = tMiddleCategories.tSubGroups[tInfo.eTier].strSubGroupName
			else
				self.data.craftingXP[idx]["subGroupName"] = tInfo.strName
			end
		end
	end
	self.data.tLayerData["CraftingPanel"]["Value"] = {}
	self.data.tLayerData["CraftingPanel"]["Tooltip"] = {}
	self.data.tLayerData["CraftingPanel"]["NextTooltip"] = {}
	for i in pairs(self.data.craftingXP) do
		if self.data.craftingXP[i]["bIsActive"] then
			self.data.tLayerData["CraftingPanel"]["Value"][i] = {}
			self.data.tLayerData["CraftingPanel"]["Value"][i]["nXp"] = self.data.craftingXP[i]["nXp"]
			self.data.tLayerData["CraftingPanel"]["Value"][i]["nXpForNextTier"] = self.data.craftingXP[i]["nXpForNextTier"]
			local nextTooltip=""
			local tooltip=""
			local nNext=0
			if i==table.getn(self.data.craftingXP) then
				nNext = 1
			else
				nNext = i+1
			end
			nextTooltip= "<P Font=\"CRB_Button\"><A TextColor=\"AttributeStamina\">Switch to <T TextColor=\"ChannelZone\">"..self.data.craftingXP[nNext]["strName"].."</T> instead of <T TextColor=\"ChannelZone\">"..self.data.craftingXP[i]["strName"].."</T>.</A></P>"
			self.data.tLayerData["CraftingPanel"]["NextTooltip"][i] = nextTooltip
			local percXp = tonumber(self.data.craftingXP[i]["nXp"]) / tonumber(self.data.craftingXP[i]["nXpForNextTier"]) * 100
			if percXp < 0 then percXp = 0 end
			tooltip = "<P Font=\"CRB_Button\"><A TextColor=\"UI_WindowTextDefault\">".. self.data.craftingXP[i]["subGroupName"].. ": </A><T TextColor=\"ChannelZone\">".. self.data.craftingXP[i]["nXp"].. " / ".. self.data.craftingXP[i]["nXpForNextTier"].. " XP ("..round(percXp,1).. "%) </T></P>"
			self.data.tLayerData["CraftingPanel"]["Tooltip"][i] = tooltip 
		end
	end

	--UPDATE PANEL TEXT
	
	if self.wndMain and not self.wndMain:IsShown() and not self.bHideMainWindow then
		self.wndMain:Invoke()
	elseif not self.wndMain or self.wndMain == nil then
		self:CreateWindows()
		return
	end
	
	self:UpdatePanel()
end

function NexusPanel:OnTimerTick()
	self.data.totalTicks = self.data.totalTicks + 1
	--Update DATA
	self:UpdateperTick()
end

function NexusPanel:OnPlayedTime(strCreationDate, strPlayedTime, strPlayedLevelTime, strPlayedSessionTime, dateCreation, nSecondsPlayed, nSecondsLevel, nSecondsSession)
	self.data.nSecondsPlayed = nSecondsPlayed
	self.data.nLevelStart = nSecondsPlayed-nSecondsLevel
end

function NexusPanel:ShowOptions()
	if self.wndOptionsContainer ~= nil and self.wndOptionsContainer then
		if self.wndOptionsContainer:IsShown() then
			self.wndOptionsContainer:Show(false)
		else
			self.wndOptionsContainer:Show(true)
		end
	end
end

function NexusPanel:ResetData()
	self.data.totalTicks = 0
	self.data.totalElderPointsGainedThisSession = 0
	self.data.totalFPS = 0
	self.data.totalLatency = 0
	self.data.totalMoneyLootedThisSession = 0
	self.data.totalPingTime = 0
	self.data.totalXPGainedThisSession = 0
	self.data.timeForLevelUp = 0
	self.data.timeForNextGem = 0
	self.data.nSessionStart = os.time()
	self:WriteChat("NexusPanel: Session resetted!")
end

function NexusPanel:WriteChat(message)
	local chatChannel = 23
	ChatSystemLib.PostOnChannel(chatChannel, message, "")
end

--Size/Anchor/Style Functions

function NexusPanel:SetAnchor(anchor)
	if self.wndMain and self.wndMain:IsShown() then
		l,t,r,b = self.wndMain:GetAnchorOffsets()
		w, h = Apollo.GetScreenSize()
		if anchor == "left" then
			self.wndMain:SetAnchorOffsets(0,t,(r-l),b)
		elseif anchor == "center" then
			dif = w - (r-l)
			self.wndMain:SetAnchorOffsets(dif/2,t,(r-l)+(dif/2),b)
		elseif anchor == "right" then
			self.wndMain:SetAnchorOffsets(w-(r-l),t,w,b)
		end
		
		local OptInterface = Apollo.GetAddon("OptionsInterface")
		if OptInterface then
			OptInterface:UpdateTrackedWindow(self.wndMain)
		end
	end
end

---MATH AND MORE FUNCTIONS
function round(num, idp)
	if num then
  		local mult = 10^(idp or 0)
  		return math.floor(num * mult + 0.5) / mult
	else
		return 0
	end
end

function formatTime(nTimeLeft)
	if type(nTimeLeft) ~= "number" then return nTimeLeft end
	local nDays, nHours, nMinutes, nSeconds = math.floor(nTimeLeft / 86400), math.floor((nTimeLeft % 86400) / 3600), math.floor((nTimeLeft % 3600) / 60), nTimeLeft % 60;

	if nDays ~= 0 then
		return ("%dd %dh %dm %ds"):format(nDays, nHours, nMinutes, nSeconds)
	elseif nHours ~= 0 then
		return ("%dh %dm %ds"):format(nHours, nMinutes, nSeconds)
	elseif nMinutes ~= 0 then
		return ("%dm %ds"):format(nMinutes, nSeconds)
	else
		return ("%ds"):format(nSeconds)
	end
end

function split(str, pat)
   local t = {}  -- NOTE: use {n = 0} in Lua-5.0
   local fpat = "(.-)" .. pat
   local last_end = 1
   local s, e, cap = str:find(fpat, 1)
   while s do
      if s ~= 1 or cap ~= "" then
	 table.insert(t,cap)
      end
      last_end = e+1
      s, e, cap = str:find(fpat, last_end)
   end
   if last_end <= #str then
      cap = str:sub(last_end)
      table.insert(t, cap)
   end
   return t
end
-----------------------------------------------------------------------------------------------
-- NexusPanelForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function NexusPanel:OnOK()
	self.wndMain:Close()
end

-- when the Cancel button is clicked
function NexusPanel:OnCancel()
	self.wndMain:Close()
end

function NexusPanel:OnPanelMouseEnter()
	--if self.config.chkbShowModeMouseOver then
	--	self.wndMain:SetOpacity(1)
		--self.wndMain:Show(true)
	--end
end

function NexusPanel:OnPanelMouseExit()
	--if self.config.chkbShowModeMouseOver then
	--	self.wndMain:SetOpacity(0)
		--self.wndMain:Show(false)
	--end
end

function NexusPanel:OnbtnOptionsButtonSignal( wndHandler, wndControl, eMouseButton )
	self.wndOptionsContainer:Show(wndControl:IsChecked())
end

-----------------------------------------------------------------------------------------------
-- SubPanels Functions
-----------------------------------------------------------------------------------------------
function NexusPanel:OnNextButtonClicked(wndHandler, wndControl, eMouseButton)
	local strSubPanelName = wndControl:GetParent():GetName()
	local nTotalLayers = 1
	if self.data.tLayerData[strSubPanelName]["Value"]["LevelGame"] then nTotalLayers = #self.data.tLayerData[strSubPanelName]["Value"]["LevelGame"] else nTotalLayers = #self.data.tLayerData[strSubPanelName]["Value"] end
	if self.config.layer[strSubPanelName] >= nTotalLayers then
		self.config.layer[strSubPanelName] = 1
	else
		self.config.layer[strSubPanelName] = self.config.layer[strSubPanelName] + 1
	end

	self:SetPanelVisibility()
end

function NexusPanel:OnSubPanelMouseEnter()
	--self.wndLocationPanel:FindChild("btnNext"):Show(true)
end

function NexusPanel:OnSubPanelMouseExit()
	--self.wndLocationPanel:FindChild("btnNext"):Show(false)
end

function NexusPanel:OnPostButtonClicked()
	local MailAddon = Apollo.GetAddon("Mail")
	if MailAddon then MailAddon:ToggleWindow() end
	--Event_FireGenericEvent("GenericEvent_ShowMail")
end

function NexusPanel:OnArrowButtonClicked(wndHandler, wndControl)
	local strSubPanelName = wndControl:GetParent():GetParent():GetName()
	local tSubPanel = nil
	
	for i,j in pairs(self.panel["SubPanels"]) do
		if j["Name"] == strSubPanelName then tSubPanel = j break end
	end
	local posAlt = tSubPanel["Position"]
	local nCounter=0
	if tSubPanel then
		for i,j in pairs(self.panel["SubPanels"]) do
			if wndControl:GetName() == "btnArrowLeft" then
				--links
				local bEnd = false
				if j["Position"] < tSubPanel["Position"] and j["Visible"] then
					for k = table.getn(self.panel["SubPanels"]), 1, -1 do
						if self.panel["SubPanels"][k]["Position"] < tSubPanel["Position"] and self.panel["SubPanels"][k]["Visible"] == false then
							nCounter = nCounter + 1
							bEnd = true
						elseif bEnd then
							break
						end
					end
					
					
					--jetzt tauschen
					local posNeu = posAlt - 1 - nCounter
					local tSubPanelTemp = self.panel["SubPanels"][posNeu]
					
					tSubPanel["Position"] = posNeu
					self.panel["SubPanels"][posNeu]["Position"] = posAlt
					
					table.remove(self.panel["SubPanels"],posAlt)
					table.insert(self.panel["SubPanels"],posAlt,tSubPanelTemp)
					
					table.remove(self.panel["SubPanels"],posNeu)
					table.insert(self.panel["SubPanels"],posNeu,tSubPanel)
					
					break
				end
			else
				--rechts
				local bEnd = false
				if j["Position"] > tSubPanel["Position"] and j["Visible"] then
					for k,l in pairs(self.panel["SubPanels"]) do
						if l["Position"] > tSubPanel["Position"] and l["Visible"] == false then
							nCounter = nCounter + 1
							bEnd = true
						elseif bEnd then
							break
						end
					end
					--jetzt tauschen
					local posNeu = posAlt + 1 + nCounter
					local tSubPanelTemp = self.panel["SubPanels"][posNeu]
					
					tSubPanel["Position"] = posNeu
					self.panel["SubPanels"][posNeu]["Position"] = posAlt
					
					table.remove(self.panel["SubPanels"],posAlt)
					table.insert(self.panel["SubPanels"],posAlt,tSubPanelTemp)
					
					table.remove(self.panel["SubPanels"],posNeu)
					table.insert(self.panel["SubPanels"],posNeu,tSubPanel)
					
					break
				end
			end
		end
	end
	
	--neu zeichnen
	self:SetPanelVisibility()
end

function NexusPanel:OnBagButtonClicked()
	Event_FireGenericEvent("InterfaceMenu_ToggleInventory")
end

function NexusPanel:OnClassIconClicked()
	Event_FireGenericEvent("ToggleCharacterWindow")
end
-----------------------------------------------------------------------------------------------
-- Options Container Functions
-----------------------------------------------------------------------------------------------
function NexusPanel:OnOptionsCloseClick(wndHandler, wndControl, eMouseButton )
	self.wndOptionsContainer:Show(false)
end

function NexusPanel:OnOptionsPanelWindowShow(wndHandler, wndControl)
	for i in pairs(self.panel["SubPanels"]) do
		self.wndOptionsContainer:FindChild("chkbShow".. self.panel["SubPanels"][i]["Name"]):SetCheck(self.panel["SubPanels"][i]["Visible"])
	end
	for k,v in pairs(self.config) do
		if (type(v) == "boolean") then
			if self.wndOptionsContainer:FindChild(k) ~= nil then
				self.wndOptionsContainer:FindChild(k):SetCheck(v)
			end
		end
	end	

end

function NexusPanel:OnOptionsPanelWindowHide(wndHandler, wndControl)
	--Set Position Mode off
	self.wndOptionsContainer:FindChild("btnSetPositionMode"):SetText("Edit positions")
	self:SetPositionMode(false)
end

function NexusPanel:OnSaveButtonClicked(wndHandler, wndControl, eMouseButton)
	if self.wndOptionsContainer ~= nil and self.wndOptionsContainer then
		self.wndOptionsContainer:Show(false)
	end
end

function NexusPanel:OnCheckboxChange( wndHandler, wndControl, eMouseButton )
	if wndControl:GetName() == "chkbShowModeInCombat" and not wndControl:IsChecked() and not wndControl:GetParent():FindChild("chkbShowModeOutOfCombat"):IsChecked() then wndControl:SetCheck(true) return end
	if wndControl:GetName() == "chkbShowModeOutOfCombat" and not wndControl:IsChecked() and not wndControl:GetParent():FindChild("chkbShowModeInCombat"):IsChecked() then wndControl:SetCheck(true) return end
	for i in pairs(self.panel["SubPanels"]) do
		if string.match(wndControl:GetName(),"Show"..self.panel["SubPanels"][i]["Name"]) then
			self.panel["SubPanels"][i]["Visible"] = wndControl:IsChecked()
		end
	end
	
	if self.config[wndControl:GetName()] ~= nil then
		self.config[wndControl:GetName()] = wndControl:IsChecked()
	end
	self:SetPanelVisibility()
end

function NexusPanel:OnAlignButtonClick(wndHandler)
	self:SetAnchor("center")
end
-----------------------------------------------------------------------------------------------
-- Slash Functions
-----------------------------------------------------------------------------------------------
function NexusPanel:OnSlashCommand(strCmd, strParam)
	if strParam ~= "" then
		if strParam == "show" then
			self.bHideMainWindow = false
			self.wndMain:Show(true)
		elseif strParam == "hide" then
			self.bHideMainWindow = true
			self.wndMain:Show(false)
		elseif strParam == "lock" then
			self.config.chkbLockPanel = true
		elseif strParam == "unlock" then
			self.config.chkbLockPanel = false
		elseif strParam == "config" then
			self:ShowOptions()
		elseif strParam == "reset" then
			self:ResetData()
		else
			--
			self:SetPanelVisibility()
		end
	end
end
-----------------------------------------------------------------------------------------------
-- Save/Restore Functions
-----------------------------------------------------------------------------------------------
function NexusPanel:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end
	local locWindowLocation = self.wndMain and self.wndMain:GetLocation() or self.config.tWindowLocation
	self.config.tWindowLocation = locWindowLocation and locWindowLocation:ToTable() or nil
	--local left, top, right, bottom = self.wndMain:GetAnchorOffsets()
	--self.config.tWindowLocation = {left,top,right,bottom}
	--self.config.tWindowLocation = self.wndMain:GetLocation()
	self.config.panel = self.panel
	self.config.selfdata = self.data
	self.config.lastLogin = os.date("*t")
	return self.config
end

function NexusPanel:OnRestore(eType, tSave)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return nil
	end
	if tSave and tSave.version and tSave.version == actVersion then
		self.config = tSave
		if tSave.panel then self.panel = tSave.panel end
		
		--check date
		if tSave.selfdata and tSave.lastLogin and tSave.lastLogin.year then
			local actDate = os.date("*t")
			if actDate.year == tSave.lastLogin.year and actDate.month == tSave.lastLogin.month and actDate.day == tSave.lastLogin.day and actDate.hour == tSave.lastLogin.hour then
				--in Seks umrechnen
				local lastSecs = (tSave.lastLogin.min * 60) + tSave.lastLogin.sec
				local SecsTemp = (actDate.min * 60) + actDate.sec
				if SecsTemp - 30 <= lastSecs then 
					self.data = tSave.selfdata 
				end
			end
		end
	
		if tSave.tWindowLocation then
			self.config.tWindowLocation = WindowLocation.new(tSave.tWindowLocation)
		end
	elseif tSave then
		self.config.version = actVersion
		if tSave.panel then self.panel = tSave.panel end
		for i,j in pairs(self.config.layer) do
			if tSave.layer.j ~= nil then self.config.layer.j = tSave.layer.j end
		end
		--if tSave.layer then self.layer = tSave.layer end
		if tSave.tWindowLocation then
			self.config.tWindowLocation = WindowLocation.new(tSave.tWindowLocation)
		end
	end	
end
-----------------------------------------------------------------------------------------------
-- NexusPanel Instance
-----------------------------------------------------------------------------------------------
local NexusPanelInst = NexusPanel:new()
NexusPanelInst:Init()
